"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const ramda_1 = require("ramda");
const core_1 = require("@serverless/core");
const lib_1 = require("./lib");
class CloudFront extends core_1.Component {
    static createInvalidation({ credentials, distributionId, paths, }) {
        return lib_1.createInvalidation({ credentials, distributionId, paths });
    }
    async default(inputs) {
        this.context.status('Deploying');
        inputs.region = inputs.region || 'us-east-1';
        inputs.enabled = inputs.enabled === false ? false : true;
        inputs.comment =
            inputs.comment === null || inputs.comment === undefined ? '' : String(inputs.comment);
        inputs.priceClass = ['PriceClass_All', 'PriceClass_200', 'PriceClass_100'].includes(inputs.priceClass || '')
            ? inputs.priceClass
            : 'PriceClass_All';
        this.context.debug(`Starting deployment of CloudFront distribution to the ${inputs.region} region.`);
        const cf = new aws_sdk_1.CloudFront({
            credentials: this.context.credentials.aws,
            region: inputs.region,
        });
        const s3 = new aws_sdk_1.S3({
            credentials: this.context.credentials.aws,
            region: inputs.region,
        });
        this.state.id = inputs.distributionId || this.state.id;
        if (this.state.id) {
            if (!ramda_1.equals(this.state.origins, inputs.origins) ||
                !ramda_1.equals(this.state.defaults, inputs.defaults) ||
                !ramda_1.equals(this.state.enabled, inputs.enabled) ||
                !ramda_1.equals(this.state.comment, inputs.comment) ||
                !ramda_1.equals(this.state.priceClass, inputs.priceClass)) {
                this.context.debug(`Updating CloudFront distribution of ID ${this.state.id}.`);
                this.state = await lib_1.updateCloudFrontDistribution(cf, s3, this.state.id, inputs);
            }
        }
        else {
            this.context.debug(`Creating CloudFront distribution in the ${inputs.region} region.`);
            this.state = await lib_1.createCloudFrontDistribution(cf, s3, inputs);
        }
        this.state.region = inputs.region;
        this.state.enabled = inputs.enabled;
        this.state.comment = inputs.comment;
        this.state.priceClass = inputs.priceClass;
        this.state.origins = inputs.origins;
        this.state.defaults = inputs.defaults;
        await this.save();
        this.context.debug(`CloudFront deployed successfully with URL: ${this.state.url}.`);
        return this.state;
    }
    async remove() {
        this.context.status('Removing');
        if (!this.state.id) {
            return;
        }
        const cf = new aws_sdk_1.CloudFront({
            credentials: this.context.credentials.aws,
            region: this.state.region,
        });
        this.context.debug(`Removing CloudFront distribution of ID ${this.state.id}. It could take a while.`);
        try {
            await lib_1.deleteCloudFrontDistribution(cf, this.state.id, this.context.debug);
        }
        catch (error) {
            this.context.debug(error.message);
        }
        this.state = {};
        await this.save();
        this.context.debug('CloudFront distribution was successfully removed.');
    }
}
exports.default = CloudFront;
