"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteCloudFrontDistribution = exports.updateCloudFrontDistribution = exports.createCloudFrontDistribution = void 0;
const parseInputOrigins_1 = __importDefault(require("./parseInputOrigins"));
const getDefaultCacheBehavior_1 = __importDefault(require("./getDefaultCacheBehavior"));
const createOriginAccessIdentity_1 = __importDefault(require("./createOriginAccessIdentity"));
const grantCloudFrontBucketAccess_1 = __importDefault(require("./grantCloudFrontBucketAccess"));
var createInvalidation_1 = require("./createInvalidation");
Object.defineProperty(exports, "createInvalidation", { enumerable: true, get: function () { return createInvalidation_1.default; } });
const servePrivateContentEnabled = (inputs) => { var _a; return (_a = inputs === null || inputs === void 0 ? void 0 : inputs.origins) === null || _a === void 0 ? void 0 : _a.some((origin) => origin && origin.private === true); };
const unique = (value, index, self) => self.indexOf(value) === index;
const updateBucketsPolicies = async (s3, origins, s3CanonicalUserId) => {
    const bucketNames = origins.Items.filter((origin) => origin.S3OriginConfig)
        .map((origin) => origin.Id.split('/')[0])
        .filter(unique);
    return Promise.all(bucketNames.map((bucketName) => grantCloudFrontBucketAccess_1.default(s3, bucketName, s3CanonicalUserId)));
};
exports.createCloudFrontDistribution = async (cf, s3, inputs) => {
    var _a, _b, _c;
    let originAccessIdentityId;
    let s3CanonicalUserId;
    if (servePrivateContentEnabled(inputs)) {
        ({ originAccessIdentityId, s3CanonicalUserId } = await createOriginAccessIdentity_1.default(cf));
    }
    const { Origins, CacheBehaviors } = parseInputOrigins_1.default(inputs.origins, {
        originAccessIdentityId,
    });
    if (s3CanonicalUserId) {
        await updateBucketsPolicies(s3, Origins, s3CanonicalUserId);
    }
    const createDistributionRequest = {
        DistributionConfig: {
            CallerReference: String(Date.now()),
            Comment: inputs.comment,
            Aliases: {
                Quantity: 0,
                Items: [],
            },
            Origins,
            PriceClass: inputs.priceClass,
            Enabled: inputs.enabled,
            HttpVersion: 'http2',
            DefaultCacheBehavior: getDefaultCacheBehavior_1.default(Origins.Items[0].Id, inputs.defaults),
        },
    };
    if (CacheBehaviors) {
        createDistributionRequest.DistributionConfig.CacheBehaviors = CacheBehaviors;
    }
    const res = await cf.createDistribution(createDistributionRequest).promise();
    return {
        id: (_a = res === null || res === void 0 ? void 0 : res.Distribution) === null || _a === void 0 ? void 0 : _a.Id,
        arn: (_b = res === null || res === void 0 ? void 0 : res.Distribution) === null || _b === void 0 ? void 0 : _b.ARN,
        url: `https://${(_c = res === null || res === void 0 ? void 0 : res.Distribution) === null || _c === void 0 ? void 0 : _c.DomainName}`,
    };
};
exports.updateCloudFrontDistribution = async (cf, s3, distributionId, inputs) => {
    var _a, _b, _c;
    const distributionConfigResponse = await cf
        .getDistributionConfig({ Id: distributionId })
        .promise();
    if (!distributionConfigResponse.DistributionConfig) {
        throw new Error('Could not get a distribution config');
    }
    let s3CanonicalUserId;
    let originAccessIdentityId;
    if (servePrivateContentEnabled(inputs)) {
        ({ originAccessIdentityId, s3CanonicalUserId } = await createOriginAccessIdentity_1.default(cf));
    }
    const { Origins, CacheBehaviors } = parseInputOrigins_1.default(inputs.origins, {
        originAccessIdentityId,
    });
    if (s3CanonicalUserId) {
        await updateBucketsPolicies(s3, Origins, s3CanonicalUserId);
    }
    const updateDistributionRequest = {
        Id: distributionId,
        IfMatch: distributionConfigResponse.ETag,
        DistributionConfig: {
            ...distributionConfigResponse.DistributionConfig,
            PriceClass: inputs.priceClass,
            Enabled: inputs.enabled,
            Comment: inputs.comment,
            DefaultCacheBehavior: getDefaultCacheBehavior_1.default(Origins.Items[0].Id, inputs.defaults),
            Origins,
        },
    };
    const origins = updateDistributionRequest.DistributionConfig.Origins;
    const existingOriginIds = origins.Items.map((origin) => origin.Id);
    Origins.Items.forEach((inputOrigin) => {
        const originIndex = existingOriginIds.indexOf(inputOrigin.Id);
        if (originIndex > -1) {
            origins.Items.splice(originIndex, 1, inputOrigin);
        }
        else {
            origins.Items.push(inputOrigin);
            origins.Quantity += 1;
        }
    });
    if (CacheBehaviors) {
        updateDistributionRequest.DistributionConfig.CacheBehaviors = CacheBehaviors;
    }
    const res = await cf.updateDistribution(updateDistributionRequest).promise();
    return {
        id: (_a = res === null || res === void 0 ? void 0 : res.Distribution) === null || _a === void 0 ? void 0 : _a.Id,
        arn: (_b = res === null || res === void 0 ? void 0 : res.Distribution) === null || _b === void 0 ? void 0 : _b.ARN,
        url: `https://${(_c = res === null || res === void 0 ? void 0 : res.Distribution) === null || _c === void 0 ? void 0 : _c.DomainName}`,
    };
};
const disableCloudFrontDistribution = async (cf, distributionId, debug) => {
    const distributionConfigResponse = await cf
        .getDistributionConfig({ Id: distributionId })
        .promise();
    if (!distributionConfigResponse.DistributionConfig) {
        throw new Error('Could not get a distribution config');
    }
    const updateDistributionRequest = {
        Id: distributionId,
        IfMatch: distributionConfigResponse.ETag,
        DistributionConfig: {
            ...distributionConfigResponse.DistributionConfig,
            Enabled: false,
        },
    };
    const res = await cf.updateDistribution(updateDistributionRequest).promise();
    debug('Waiting for the CloudFront distribution changes to be deployed.');
    await cf.waitFor('distributionDeployed', { Id: distributionId }).promise();
    return res;
};
exports.deleteCloudFrontDistribution = async (cf, distributionId, debug) => {
    try {
        const res = await cf.getDistributionConfig({ Id: distributionId }).promise();
        const params = { Id: distributionId, IfMatch: res.ETag };
        await cf.deleteDistribution(params).promise();
    }
    catch (e) {
        if (e.code === 'DistributionNotDisabled') {
            await disableCloudFrontDistribution(cf, distributionId, debug);
        }
        else {
            throw e;
        }
    }
};
