"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.REQUEST_LAMBDA_CODE_DIR = exports.BUILD_DIR = void 0;
const core_1 = require("@serverless/core");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const aws_lambda_builder_1 = __importDefault(require("@next-deploy/aws-lambda-builder"));
const aws_s3_1 = __importDefault(require("@next-deploy/aws-s3"));
const aws_cloudfront_1 = __importDefault(require("@next-deploy/aws-cloudfront"));
const utils_1 = require("./utils");
exports.BUILD_DIR = '.next-deploy-build';
exports.REQUEST_LAMBDA_CODE_DIR = `${exports.BUILD_DIR}/request-lambda`;
class Aws extends core_1.Component {
    async default(inputs = {}) {
        await this.build(inputs);
        return this.deploy(inputs);
    }
    readRequestLambdaBuildManifest(nextConfigPath) {
        return fs_extra_1.readJSON(path_1.join(nextConfigPath, `${exports.REQUEST_LAMBDA_CODE_DIR}/manifest.json`));
    }
    validatePathPatterns(pathPatterns, buildManifest) {
        const stillToMatch = new Set(pathPatterns);
        if (stillToMatch.size !== pathPatterns.length) {
            throw Error('Duplicate path declared in cloudfront configuration');
        }
        stillToMatch.delete('api/*');
        stillToMatch.delete('static/*');
        stillToMatch.delete('_next/static/*');
        for (const path of stillToMatch) {
            if (/^(\/?api\/.*|\/?api)$/.test(path)) {
                throw Error(`Setting custom cache behaviour for api/ route "${path}" is not supported`);
            }
        }
        const manifestRegex = [];
        const manifestPaths = new Set();
        const ssrDynamic = buildManifest.pages.ssr.dynamic || {};
        const ssrNonDynamic = buildManifest.pages.ssr.nonDynamic || {};
        const htmlDynamic = buildManifest.pages.html.dynamic || {};
        const htmlNonDynamic = buildManifest.pages.html.nonDynamic || {};
        Object.entries({
            ...ssrDynamic,
            ...htmlDynamic,
        }).map(([, { regex }]) => {
            manifestRegex.push(new RegExp(regex));
        });
        Object.entries({
            ...ssrNonDynamic,
            ...htmlNonDynamic,
        }).map(([path]) => {
            manifestPaths.add(path);
        });
        manifestRegex.forEach((re) => {
            for (const path of stillToMatch) {
                if (re.test(path)) {
                    stillToMatch.delete(path);
                }
            }
        });
        for (const pathToMatch of stillToMatch) {
            for (const path of manifestPaths) {
                if (new RegExp(pathToMatch).test(path)) {
                    stillToMatch.delete(pathToMatch);
                }
            }
        }
        if (stillToMatch.size > 0) {
            throw Error(`CloudFront input failed validation. Could not find next.js pages for "${[
                ...stillToMatch,
            ]}"`);
        }
    }
    async build({ build, nextConfigDir } = {}) {
        this.context.status('Building');
        const nextConfigPath = nextConfigDir ? path_1.resolve(nextConfigDir) : process.cwd();
        const builder = new aws_lambda_builder_1.default(nextConfigPath, path_1.join(nextConfigPath, exports.BUILD_DIR), {
            cmd: (build === null || build === void 0 ? void 0 : build.cmd) || 'node_modules/.bin/next',
            cwd: (build === null || build === void 0 ? void 0 : build.cwd) ? path_1.resolve(build.cwd) : nextConfigPath,
            args: (build === null || build === void 0 ? void 0 : build.args) || ['build'],
        });
        await builder.build(this.context.instance.debugMode ? this.context.debug : undefined);
    }
    async deploy({ nextConfigDir, nextStaticDir, bucketRegion, bucketName, cloudfront: cloudfrontInput, policy, publicDirectoryCache, domain, domainType, stage, ...inputs } = {}) {
        this.context.status('Deploying');
        const nextConfigPath = nextConfigDir ? path_1.resolve(nextConfigDir) : process.cwd();
        const nextStaticPath = nextStaticDir ? path_1.resolve(nextStaticDir) : nextConfigPath;
        const { defaults: cloudFrontDefaultsInputs, origins: cloudFrontOriginsInputs, priceClass: cloudFrontPriceClassInputs, ...cloudFrontOtherInputs } = cloudfrontInput || {};
        const cloudFrontDefaults = cloudFrontDefaultsInputs || {};
        const calculatedBucketRegion = bucketRegion || 'us-east-1';
        const stageBucket = await utils_1.load('@next-deploy/aws-s3', this, 'StageStateStorage');
        const stageStateBucketName = (typeof stage !== 'boolean' && (stage === null || stage === void 0 ? void 0 : stage.bucketName)) || 'next-deploy-environments';
        const isSyncStateVersioned = typeof stage !== 'boolean' && (stage === null || stage === void 0 ? void 0 : stage.versioned);
        const stageName = (stage === null || stage === void 0 ? void 0 : stage.name) || 'local';
        const canSyncStageState = stageName !== 'local';
        if (canSyncStageState) {
            await stageBucket.default({
                accelerated: true,
                name: stageStateBucketName,
                region: calculatedBucketRegion,
            });
            await aws_s3_1.default.syncStageStateDirectory({
                name: stageName,
                bucketName: stageStateBucketName,
                versioned: isSyncStateVersioned,
                nextConfigDir: nextConfigPath,
                credentials: this.context.credentials.aws,
            });
        }
        const [bucket, cloudfront, requestEdgeLambda, defaultBuildManifest] = await Promise.all([
            utils_1.load('@next-deploy/aws-s3', this, 'StaticStorage'),
            utils_1.load('@next-deploy/aws-cloudfront', this),
            utils_1.load('@next-deploy/aws-lambda', this, 'RequestEdgeLambda'),
            this.readRequestLambdaBuildManifest(nextConfigPath),
        ]);
        const bucketOutputs = await bucket.default({
            accelerated: true,
            name: bucketName,
            region: calculatedBucketRegion,
        });
        const bucketUrl = `http://${bucketOutputs.name}.s3.${calculatedBucketRegion}.amazonaws.com`;
        await aws_s3_1.default.uploadStaticAssets({
            bucketName: bucketOutputs.name,
            nextConfigDir: nextConfigPath,
            nextStaticDir: nextStaticPath,
            credentials: this.context.credentials.aws,
            publicDirectoryCache: publicDirectoryCache,
        });
        const expandRelativeUrls = (origin) => {
            const originUrl = typeof origin === 'string' ? origin : origin.url;
            const fullOriginUrl = originUrl.charAt(0) === '/' ? `${bucketUrl}${originUrl}` : originUrl;
            if (typeof origin === 'string') {
                return fullOriginUrl;
            }
            else {
                return {
                    ...origin,
                    url: fullOriginUrl,
                };
            }
        };
        let inputOrigins = [];
        if (cloudFrontOriginsInputs) {
            const origins = cloudFrontOriginsInputs;
            inputOrigins = origins.map(expandRelativeUrls);
        }
        const cloudFrontOrigins = [
            {
                url: bucketUrl,
                private: true,
                pathPatterns: {
                    '_next/static/*': {
                        ttl: 86400,
                        forward: {
                            headers: 'none',
                            cookies: 'none',
                            queryString: false,
                        },
                    },
                    'static/*': {
                        ttl: 86400,
                        forward: {
                            headers: 'none',
                            cookies: 'none',
                            queryString: false,
                        },
                    },
                },
            },
            ...inputOrigins,
        ];
        const getLambdaInputValue = (inputKey, lambdaType, defaultValue) => {
            const inputValue = inputs[inputKey];
            if (typeof inputValue === 'string' || typeof inputValue === 'number') {
                return inputValue;
            }
            if (!inputValue) {
                return defaultValue;
            }
            return inputValue[lambdaType] || defaultValue;
        };
        const defaultEdgeLambdaInput = {
            handler: 'index.handler',
            code: path_1.join(nextConfigPath, exports.REQUEST_LAMBDA_CODE_DIR),
            role: {
                service: ['lambda.amazonaws.com', 'edgelambda.amazonaws.com'],
                policy: {
                    arn: policy || 'arn:aws:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole',
                },
            },
            memory: getLambdaInputValue('memory', 'requestLambda', 512),
            timeout: getLambdaInputValue('timeout', 'requestLambda', 10),
            runtime: getLambdaInputValue('runtime', 'requestLambda', 'nodejs12.x'),
            name: getLambdaInputValue('name', 'requestLambda', undefined),
            description: getLambdaInputValue('description', 'requestLambda', 'Request handler for the Next CloudFront distribution.'),
        };
        const requestEdgeLambdaOutputs = await requestEdgeLambda.default(defaultEdgeLambdaInput);
        const requestEdgeLambdaPublishOutputs = await requestEdgeLambda.publishVersion();
        this.validatePathPatterns(Object.keys(cloudFrontOtherInputs), defaultBuildManifest);
        Object.entries(cloudFrontOtherInputs).map(([path, config]) => {
            const edgeConfig = {
                ...(config['lambda@edge'] || {}),
            };
            if (!['static/*', '_next/*'].includes(path)) {
                edgeConfig['origin-request'] = `${requestEdgeLambdaOutputs.arn}:${requestEdgeLambdaPublishOutputs.version}`;
            }
            cloudFrontOrigins[0].pathPatterns[path] = {
                ...cloudFrontOrigins[0].pathPatterns[path],
                ...config,
                'lambda@edge': {
                    ...(cloudFrontOrigins[0].pathPatterns[path] &&
                        cloudFrontOrigins[0].pathPatterns[path]['lambda@edge']),
                    ...edgeConfig,
                },
            };
        });
        cloudFrontOrigins[0].pathPatterns['_next/data/*'] = {
            ttl: 0,
            allowedHttpMethods: ['HEAD', 'GET'],
            'lambda@edge': {
                'origin-request': `${requestEdgeLambdaOutputs.arn}:${requestEdgeLambdaPublishOutputs.version}`,
            },
        };
        const defaultLambdaAtEdgeConfig = {
            ...(cloudFrontDefaults['lambda@edge'] || {}),
        };
        const cloudFrontOutputs = await cloudfront.default({
            defaults: {
                ttl: 0,
                ...cloudFrontDefaults,
                forward: {
                    cookies: 'all',
                    queryString: true,
                    ...cloudFrontDefaults.forward,
                },
                allowedHttpMethods: ['HEAD', 'DELETE', 'POST', 'GET', 'OPTIONS', 'PUT', 'PATCH'],
                'lambda@edge': {
                    ...defaultLambdaAtEdgeConfig,
                    'origin-request': `${requestEdgeLambdaOutputs.arn}:${requestEdgeLambdaPublishOutputs.version}`,
                },
                compress: true,
            },
            origins: cloudFrontOrigins,
            ...(cloudFrontPriceClassInputs && {
                priceClass: cloudFrontPriceClassInputs,
            }),
        });
        let appUrl = cloudFrontOutputs.url;
        await aws_cloudfront_1.default.createInvalidation({
            distributionId: cloudFrontOutputs.id,
            credentials: this.context.credentials.aws,
        });
        const { domain: calculatedDomain, subdomain } = utils_1.getDomains(domain);
        if (calculatedDomain && subdomain) {
            const domainComponent = await utils_1.load('@next-deploy/aws-domain', this);
            const domainOutputs = await domainComponent.default({
                privateZone: false,
                domain: calculatedDomain,
                subdomains: {
                    [subdomain]: cloudFrontOutputs,
                },
                domainType: domainType || 'both',
                defaultCloudfrontInputs: cloudFrontDefaults,
            });
            appUrl = domainOutputs.domains[0];
        }
        if (canSyncStageState) {
            await aws_s3_1.default.syncStageStateDirectory({
                name: stageName,
                bucketName: stageStateBucketName,
                versioned: isSyncStateVersioned,
                nextConfigDir: nextConfigPath,
                credentials: this.context.credentials.aws,
                syncTo: true,
            });
        }
        return {
            appUrl,
            bucketName: bucketOutputs.name,
        };
    }
    async remove() {
        const [bucket, cloudfront, domain] = await Promise.all([
            utils_1.load('@next-deploy/aws-s3', this, 'StaticStorage'),
            utils_1.load('@next-deploy/aws-cloudfront', this),
            utils_1.load('@next-deploy/aws-domain', this),
        ]);
        await Promise.all([bucket.remove(), cloudfront.remove(), domain.remove()]);
        this.context.log('You will need to manually delete your deployed lambda functions as it may take a while (hours) for them to detech from CloudFront.');
    }
}
exports.default = Aws;
