"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeDomainFromCloudFrontDistribution = exports.addDomainToCloudfrontDistribution = exports.removeCloudFrontDomainDnsRecords = exports.configureDnsForCloudFrontDistribution = exports.validateCertificate = exports.createCertificate = exports.getCertificateArnByDomain = exports.describeCertificateByArn = exports.getDomainHostedZoneId = exports.prepareSubdomains = exports.getClients = void 0;
const aws_sdk_1 = require("aws-sdk");
const core_1 = require("@serverless/core");
const DEFAULT_MINIMUM_PROTOCOL_VERSION = 'TLSv1.2_2018';
const HOSTED_ZONE_ID = 'Z2FDTNDATAQYW2';
exports.getClients = (credentials, region = 'us-east-1') => {
    const route53 = new aws_sdk_1.Route53({
        credentials,
        region,
    });
    const acm = new aws_sdk_1.ACM({
        credentials,
        region: 'us-east-1',
    });
    const cf = new aws_sdk_1.CloudFront({
        credentials,
        region,
    });
    return {
        route53,
        acm,
        cf,
    };
};
exports.prepareSubdomains = (inputs) => {
    const subdomains = [];
    for (const subdomain in inputs.subdomains || {}) {
        const domainObj = {};
        domainObj.domain = `${subdomain}.${inputs.domain}`;
        if (inputs.subdomains[subdomain].url.includes('cloudfront')) {
            domainObj.distributionId = inputs.subdomains[subdomain].id;
            domainObj.url = inputs.subdomains[subdomain].url;
            domainObj.type = 'awsCloudFront';
        }
        subdomains.push(domainObj);
    }
    return subdomains;
};
exports.getDomainHostedZoneId = async (route53, domain, privateZone) => {
    const hostedZonesRes = await route53.listHostedZonesByName().promise();
    const hostedZone = hostedZonesRes.HostedZones.find((zone) => { var _a; return ((_a = zone === null || zone === void 0 ? void 0 : zone.Config) === null || _a === void 0 ? void 0 : _a.PrivateZone) === privateZone && zone.Name.includes(domain); });
    if (!hostedZone) {
        throw Error(`Domain ${domain} was not found in your AWS account. Please purchase it from Route53 first then try again.`);
    }
    return hostedZone.Id.replace('/hostedzone/', '');
};
exports.describeCertificateByArn = async (acm, certificateArn) => {
    const certificate = await acm.describeCertificate({ CertificateArn: certificateArn }).promise();
    return certificate && certificate.Certificate ? certificate.Certificate : null;
};
exports.getCertificateArnByDomain = async (acm, domain) => {
    const listRes = await acm.listCertificates().promise();
    if (!listRes.CertificateSummaryList) {
        throw new Error('Could not get a list of certificates');
    }
    for (const certificate of listRes.CertificateSummaryList) {
        if (certificate.DomainName === domain && certificate.CertificateArn) {
            if (domain.startsWith('www.')) {
                const nakedDomain = domain.replace('wwww.', '');
                const certDetail = await exports.describeCertificateByArn(acm, certificate.CertificateArn);
                if (!(certDetail === null || certDetail === void 0 ? void 0 : certDetail.DomainValidationOptions)) {
                    throw new Error('Could not get a domain validation options');
                }
                const nakedDomainCert = certDetail.DomainValidationOptions.find(({ DomainName }) => DomainName === nakedDomain);
                if (!nakedDomainCert) {
                    continue;
                }
            }
            return certificate.CertificateArn;
        }
    }
    return null;
};
exports.createCertificate = async (acm, domain) => {
    const wildcardSubDomain = `*.${domain}`;
    const params = {
        DomainName: domain,
        SubjectAlternativeNames: [domain, wildcardSubDomain],
        ValidationMethod: 'DNS',
    };
    const res = await acm.requestCertificate(params).promise();
    return res.CertificateArn;
};
exports.validateCertificate = async (acm, route53, certificate, domain, domainHostedZoneId) => {
    let readinessCheckCount = 16;
    let statusCheckCount = 16;
    const checkReadiness = async function () {
        if (readinessCheckCount < 1) {
            throw new Error('Your newly created AWS ACM Certificate is taking a while to initialize.  Please try running this component again in a few minutes.');
        }
        const cert = await exports.describeCertificateByArn(acm, certificate.CertificateArn);
        if (!(cert === null || cert === void 0 ? void 0 : cert.DomainValidationOptions)) {
            throw new Error(`Could not get a certificate by ${certificate.CertificateArn}`);
        }
        cert.DomainValidationOptions.forEach((option) => {
            if (domain === option.DomainName && option.ResourceRecord) {
                return option.ResourceRecord;
            }
        });
        readinessCheckCount--;
        await core_1.utils.sleep(5000);
        return await checkReadiness();
    };
    const validationResourceRecord = await checkReadiness();
    const checkRecordsParams = {
        HostedZoneId: domainHostedZoneId,
        MaxItems: '10',
        StartRecordName: validationResourceRecord.Name,
    };
    const existingRecords = await route53.listResourceRecordSets(checkRecordsParams).promise();
    if (!existingRecords.ResourceRecordSets.length) {
        const recordParams = {
            HostedZoneId: domainHostedZoneId,
            ChangeBatch: {
                Changes: [
                    {
                        Action: 'UPSERT',
                        ResourceRecordSet: {
                            Name: validationResourceRecord.Name,
                            Type: validationResourceRecord.Type,
                            TTL: 300,
                            ResourceRecords: [
                                {
                                    Value: validationResourceRecord.Value,
                                },
                            ],
                        },
                    },
                ],
            },
        };
        await route53.changeResourceRecordSets(recordParams).promise();
    }
    const checkStatus = async function () {
        if (statusCheckCount < 1) {
            throw new Error('Your newly validated AWS ACM Certificate is taking a while to register as valid.  Please try running this component again in a few minutes.');
        }
        const cert = await exports.describeCertificateByArn(acm, certificate.CertificateArn);
        if ((cert === null || cert === void 0 ? void 0 : cert.Status) !== 'ISSUED') {
            statusCheckCount--;
            await core_1.utils.sleep(10000);
            return await checkStatus();
        }
    };
    await checkStatus();
};
exports.configureDnsForCloudFrontDistribution = async (route53, subdomain, domainHostedZoneId, distributionUrl, domainType) => {
    const dnsRecordParams = {
        HostedZoneId: domainHostedZoneId,
        ChangeBatch: {
            Changes: [],
        },
    };
    if (!subdomain.domain.startsWith('www.') || domainType !== 'apex') {
        dnsRecordParams.ChangeBatch.Changes.push({
            Action: 'UPSERT',
            ResourceRecordSet: {
                Name: subdomain.domain,
                Type: 'A',
                AliasTarget: {
                    HostedZoneId: HOSTED_ZONE_ID,
                    DNSName: distributionUrl,
                    EvaluateTargetHealth: false,
                },
            },
        });
    }
    if (subdomain.domain.startsWith('www.') && domainType !== 'www') {
        dnsRecordParams.ChangeBatch.Changes.push({
            Action: 'UPSERT',
            ResourceRecordSet: {
                Name: subdomain.domain.replace('www.', ''),
                Type: 'A',
                AliasTarget: {
                    HostedZoneId: HOSTED_ZONE_ID,
                    DNSName: distributionUrl,
                    EvaluateTargetHealth: false,
                },
            },
        });
    }
    return route53.changeResourceRecordSets(dnsRecordParams).promise();
};
exports.removeCloudFrontDomainDnsRecords = async (route53, domain, domainHostedZoneId, distributionUrl) => {
    const params = {
        HostedZoneId: domainHostedZoneId,
        ChangeBatch: {
            Changes: [
                {
                    Action: 'DELETE',
                    ResourceRecordSet: {
                        Name: domain,
                        Type: 'A',
                        AliasTarget: {
                            HostedZoneId: HOSTED_ZONE_ID,
                            DNSName: distributionUrl,
                            EvaluateTargetHealth: false,
                        },
                    },
                },
            ],
        },
    };
    try {
        await route53.changeResourceRecordSets(params).promise();
    }
    catch (e) {
        if (e.code !== 'InvalidChangeBatch') {
            throw e;
        }
    }
};
exports.addDomainToCloudfrontDistribution = async (cf, subdomain, certificateArn, domainType, { viewerCertificate }) => {
    var _a, _b, _c;
    const distributionConfigResponse = await cf
        .getDistributionConfig({ Id: subdomain.distributionId })
        .promise();
    if (!distributionConfigResponse.DistributionConfig) {
        throw new Error('Could not get a distribution config');
    }
    const updateDistributionRequest = {
        IfMatch: distributionConfigResponse.ETag,
        Id: subdomain.distributionId,
        DistributionConfig: {
            ...distributionConfigResponse.DistributionConfig,
            Aliases: {
                Quantity: 1,
                Items: [subdomain.domain],
            },
            ViewerCertificate: {
                ACMCertificateArn: (viewerCertificate === null || viewerCertificate === void 0 ? void 0 : viewerCertificate.ACMCertificateArn) || certificateArn,
                SSLSupportMethod: (viewerCertificate === null || viewerCertificate === void 0 ? void 0 : viewerCertificate.SSLSupportMethod) || 'sni-only',
                MinimumProtocolVersion: (viewerCertificate === null || viewerCertificate === void 0 ? void 0 : viewerCertificate.minimumProtocolVersion) || DEFAULT_MINIMUM_PROTOCOL_VERSION,
                Certificate: certificateArn,
                CertificateSource: 'acm',
            },
        },
    };
    if (subdomain.domain.startsWith('www.')) {
        if (domainType === 'apex') {
            updateDistributionRequest.DistributionConfig.Aliases.Items = [
                `${subdomain.domain.replace('www.', '')}`,
            ];
        }
        else if (domainType !== 'www') {
            updateDistributionRequest.DistributionConfig.Aliases.Quantity = 2;
            updateDistributionRequest.DistributionConfig.Aliases.Items.push(`${subdomain.domain.replace('www.', '')}`);
        }
    }
    const res = await cf.updateDistribution(updateDistributionRequest).promise();
    return {
        id: (_a = res === null || res === void 0 ? void 0 : res.Distribution) === null || _a === void 0 ? void 0 : _a.Id,
        arn: (_b = res === null || res === void 0 ? void 0 : res.Distribution) === null || _b === void 0 ? void 0 : _b.ARN,
        url: (_c = res === null || res === void 0 ? void 0 : res.Distribution) === null || _c === void 0 ? void 0 : _c.DomainName,
    };
};
exports.removeDomainFromCloudFrontDistribution = async (cf, subdomain) => {
    var _a, _b, _c;
    const distributionConfigResponse = await cf
        .getDistributionConfig({ Id: subdomain.distributionId })
        .promise();
    if (!distributionConfigResponse.DistributionConfig) {
        throw new Error('Could not get a distribution config');
    }
    const updateDistributionRequest = {
        Id: subdomain.distributionId,
        IfMatch: distributionConfigResponse.ETag,
        DistributionConfig: {
            ...distributionConfigResponse.DistributionConfig,
            Aliases: {
                Quantity: 0,
                Items: [],
            },
            ViewerCertificate: {
                SSLSupportMethod: 'sni-only',
                MinimumProtocolVersion: DEFAULT_MINIMUM_PROTOCOL_VERSION,
            },
        },
    };
    const res = await cf.updateDistribution(updateDistributionRequest).promise();
    return {
        id: (_a = res === null || res === void 0 ? void 0 : res.Distribution) === null || _a === void 0 ? void 0 : _a.Id,
        arn: (_b = res === null || res === void 0 ? void 0 : res.Distribution) === null || _b === void 0 ? void 0 : _b.ARN,
        url: (_c = res === null || res === void 0 ? void 0 : res.Distribution) === null || _c === void 0 ? void 0 : _c.DomainName,
    };
};
