"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ramda_1 = require("ramda");
const aws_sdk_1 = require("aws-sdk");
const core_1 = require("@serverless/core");
const utils_1 = require("./utils");
const defaults = {
    service: 'lambda.amazonaws.com',
    policy: {
        arn: 'arn:aws:iam::aws:policy/AdministratorAccess',
    },
    region: 'us-east-1',
};
class IamRole extends core_1.Component {
    async default(inputs = {}) {
        inputs = ramda_1.mergeDeepRight(defaults, inputs);
        const iam = new aws_sdk_1.IAM({ region: inputs.region, credentials: this.context.credentials.aws });
        this.context.status('Deploying');
        inputs.name = this.state.name || this.context.resourceId();
        this.context.debug(`Syncing role ${inputs.name} in region ${inputs.region}.`);
        const prevRole = await utils_1.getRole({ iam, name: inputs.name });
        if (!prevRole) {
            this.context.debug(`Creating role ${inputs.name}.`);
            this.context.status('Creating');
            inputs.arn = await utils_1.createRole({
                iam,
                name: inputs.name,
                service: inputs.service,
                policy: inputs.policy,
            });
        }
        else {
            inputs.arn = prevRole.arn;
            if (utils_1.inputsChanged(prevRole, inputs)) {
                this.context.status(`Updating`);
                if (prevRole.service !== inputs.service) {
                    this.context.debug(`Updating service for role ${inputs.name}.`);
                    await utils_1.updateAssumeRolePolicy({
                        iam,
                        name: inputs.name,
                        service: inputs.service,
                    });
                }
                if (!ramda_1.equals(prevRole.policy, inputs.policy)) {
                    this.context.debug(`Updating policy for role ${inputs.name}.`);
                    await utils_1.removeRolePolicy({
                        iam,
                        name: inputs.name,
                        policy: inputs.policy,
                    });
                    await utils_1.addRolePolicy({
                        iam,
                        name: inputs.name,
                        policy: inputs.policy,
                    });
                }
            }
        }
        if (this.state.name && this.state.name !== inputs.name) {
            this.context.status(`Replacing`);
            this.context.debug(`Deleting/Replacing role ${inputs.name}.`);
            await utils_1.deleteRole({ iam, name: this.state.name, policy: inputs.policy });
        }
        this.state.name = inputs.name;
        this.state.arn = inputs.arn;
        this.state.service = inputs.service;
        this.state.policy = inputs.policy;
        this.state.region = inputs.region;
        await this.save();
        this.context.debug(`Saved state for role ${inputs.name}.`);
        const outputs = {
            name: inputs.name,
            arn: inputs.arn,
            service: inputs.service,
            policy: inputs.policy,
        };
        this.context.debug(`Role ${inputs.name} was successfully deployed to region ${inputs.region}.`);
        this.context.debug(`Deployed role arn is ${inputs.arn}.`);
        return outputs;
    }
    async remove() {
        this.context.status('Removing');
        if (!this.state.name) {
            this.context.debug('Aborting removal. Role name not found in state.');
            return;
        }
        const iam = new aws_sdk_1.IAM({
            region: this.state.region,
            credentials: this.context.credentials.aws,
        });
        this.context.debug(`Removing role ${this.state.name} from region ${this.state.region}.`);
        await utils_1.deleteRole({ iam, ...this.state });
        this.context.debug(`Role ${this.state.name} successfully removed from region ${this.state.region}.`);
        const outputs = {
            name: this.state.name,
            arn: this.state.arn,
            service: this.state.service,
            policy: this.state.policy,
        };
        this.state = {};
        await this.save();
        return outputs;
    }
}
exports.default = IamRole;
