"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.REQUEST_LAMBDA_CODE_DIR = void 0;
const node_file_trace_1 = require("@zeit/node-file-trace");
const execa_1 = __importDefault(require("execa"));
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const getAllFilesInDirectory_1 = __importDefault(require("./lib/getAllFilesInDirectory"));
const sortedRoutes_1 = require("./lib/sortedRoutes");
const createServerlessConfig_1 = __importDefault(require("./lib/createServerlessConfig"));
const utils_1 = require("./utils");
exports.REQUEST_LAMBDA_CODE_DIR = 'request-lambda';
const defaultBuildOptions = {
    args: [],
    cwd: process.cwd(),
    cmd: './node_modules/.bin/next',
};
class Builder {
    constructor(nextConfigDir, outputDir, buildOptions) {
        this.buildOptions = defaultBuildOptions;
        this.nextConfigDir = path_1.resolve(nextConfigDir);
        this.dotNextDir = path_1.join(this.nextConfigDir, '.next');
        this.serverlessDir = path_1.join(this.dotNextDir, 'serverless');
        this.outputDir = outputDir;
        if (buildOptions) {
            this.buildOptions = buildOptions;
        }
    }
    async readPublicFiles() {
        const dirExists = await fs_extra_1.pathExists(path_1.join(this.nextConfigDir, 'public'));
        if (dirExists) {
            return getAllFilesInDirectory_1.default(path_1.join(this.nextConfigDir, 'public'))
                .map((e) => e.replace(this.nextConfigDir, ''))
                .map((e) => e.split(path_1.sep).slice(2).join('/'));
        }
        else {
            return [];
        }
    }
    async readPagesManifest() {
        const path = path_1.join(this.serverlessDir, 'pages-manifest.json');
        const hasServerlessPageManifest = await fs_extra_1.pathExists(path);
        if (!hasServerlessPageManifest) {
            return Promise.reject("pages-manifest not found. Check if `next.config.js` target is set to 'serverless'");
        }
        const pagesManifest = await fs_extra_1.readJSON(path);
        const pagesManifestWithoutDynamicRoutes = Object.keys(pagesManifest).reduce((acc, route) => {
            if (utils_1.isDynamicRoute(route)) {
                return acc;
            }
            acc[route] = pagesManifest[route];
            return acc;
        }, {});
        const dynamicRoutedPages = Object.keys(pagesManifest).filter(utils_1.isDynamicRoute);
        const sortedDynamicRoutedPages = sortedRoutes_1.getSortedRoutes(dynamicRoutedPages);
        const sortedPagesManifest = pagesManifestWithoutDynamicRoutes;
        sortedDynamicRoutedPages.forEach((route) => (sortedPagesManifest[route] = pagesManifest[route]));
        return sortedPagesManifest;
    }
    copyLambdaHandlerDependencies(fileList, reasons, handlerDirectory) {
        return (fileList
            .filter((file) => file !== 'package.json' && (!reasons[file] || reasons[file].type !== 'initial'))
            .map((filePath) => {
            const resolvedFilePath = path_1.resolve(filePath);
            const dst = utils_1.normalizeNodeModules(path_1.relative(this.serverlessDir, resolvedFilePath));
            return fs_extra_1.copy(resolvedFilePath, path_1.join(this.outputDir, handlerDirectory, dst));
        }));
    }
    async buildRequestLambda(buildManifest) {
        const ignoreAppAndDocumentPages = (page) => {
            const pageBasename = path_1.basename(page);
            return pageBasename !== '_app.js' && pageBasename !== '_document.js';
        };
        const allServerProcessablePages = [
            ...Object.values(buildManifest.pages.ssr.nonDynamic),
            ...Object.values(buildManifest.pages.ssr.dynamic).map((entry) => entry.file),
            ...Object.values(buildManifest.pages.apis.nonDynamic),
            ...Object.values(buildManifest.pages.apis.dynamic).map((entry) => entry.file),
        ].filter(ignoreAppAndDocumentPages);
        const ssrPages = Object.values(allServerProcessablePages).map((pageFile) => path_1.join(this.serverlessDir, pageFile));
        const { fileList, reasons } = await node_file_trace_1.nodeFileTrace(ssrPages, {
            base: process.cwd(),
        });
        const copyTraces = this.copyLambdaHandlerDependencies(fileList, reasons, exports.REQUEST_LAMBDA_CODE_DIR);
        return Promise.all([
            ...copyTraces,
            fs_extra_1.copy(require.resolve('@next-deploy/aws-lambda-builder/dist/request-handler.js'), path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR, 'index.js')),
            fs_extra_1.copy(require.resolve('@next-deploy/aws-lambda-builder/dist/compat.js'), path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR, 'compat.js')),
            fs_extra_1.writeJson(path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR, 'manifest.json'), buildManifest),
            fs_extra_1.copy(path_1.join(this.serverlessDir, 'pages'), path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR, 'pages'), {
                filter: (file) => path_1.extname(file) !== '.html' && path_1.extname(file) !== '.json',
            }),
            fs_extra_1.copy(path_1.join(this.dotNextDir, 'prerender-manifest.json'), path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR, 'prerender-manifest.json')),
        ]);
    }
    async prepareBuildManifest() {
        const pagesManifest = await this.readPagesManifest();
        const buildId = await fs_extra_1.readFile(path_1.join(this.dotNextDir, 'BUILD_ID'), 'utf-8');
        const defaultBuildManifest = {
            buildId,
            pages: {
                ssr: {
                    dynamic: {},
                    nonDynamic: {},
                },
                html: {
                    dynamic: {},
                    nonDynamic: {},
                },
                apis: {
                    dynamic: {},
                    nonDynamic: {},
                },
            },
            publicFiles: {},
        };
        const ssrPages = defaultBuildManifest.pages.ssr;
        const htmlPages = defaultBuildManifest.pages.html;
        const apiPages = defaultBuildManifest.pages.apis;
        const isHtmlPage = (path) => path.endsWith('.html');
        const isApiPage = (path) => path.startsWith('pages/api');
        Object.entries(pagesManifest).forEach(([route, pageFile]) => {
            const dynamicRoute = utils_1.isDynamicRoute(route);
            const expressRoute = dynamicRoute ? utils_1.expressifyDynamicRoute(route) : null;
            if (isHtmlPage(pageFile)) {
                if (dynamicRoute) {
                    const route = expressRoute;
                    htmlPages.dynamic[route] = {
                        file: pageFile,
                        regex: utils_1.pathToRegexStr(route),
                    };
                }
                else {
                    htmlPages.nonDynamic[route] = pageFile;
                }
            }
            else if (isApiPage(pageFile)) {
                if (dynamicRoute) {
                    const route = expressRoute;
                    apiPages.dynamic[route] = {
                        file: pageFile,
                        regex: utils_1.pathToRegexStr(route),
                    };
                }
                else {
                    apiPages.nonDynamic[route] = pageFile;
                }
            }
            else if (dynamicRoute) {
                const route = expressRoute;
                ssrPages.dynamic[route] = {
                    file: pageFile,
                    regex: utils_1.pathToRegexStr(route),
                };
            }
            else {
                ssrPages.nonDynamic[route] = pageFile;
            }
        });
        const publicFiles = await this.readPublicFiles();
        publicFiles.forEach((pf) => (defaultBuildManifest.publicFiles[`/${pf}`] = pf));
        return defaultBuildManifest;
    }
    async cleanupDotNext() {
        const exists = await fs_extra_1.pathExists(this.dotNextDir);
        if (exists) {
            const fileItems = await fs_extra_1.readdir(this.dotNextDir);
            await Promise.all(fileItems
                .filter((fileItem) => fileItem !== 'cache')
                .map((fileItem) => fs_extra_1.remove(path_1.join(this.dotNextDir, fileItem))));
        }
    }
    async build(debug) {
        const { cmd, args, cwd } = { ...defaultBuildOptions, ...this.buildOptions };
        await this.cleanupDotNext();
        await fs_extra_1.emptyDir(path_1.join(this.outputDir, exports.REQUEST_LAMBDA_CODE_DIR));
        const { restoreUserConfig } = await createServerlessConfig_1.default(cwd, path_1.join(this.nextConfigDir));
        try {
            if (debug) {
                const { stdout: nextVersion } = await execa_1.default(cmd, ['--version'], {
                    cwd,
                });
                debug(`Starting a new build with ${nextVersion}`);
                console.log();
            }
            const subprocess = execa_1.default(cmd, args, {
                cwd,
                env: {
                    NODE_OPTIONS: '--max_old_space_size=3000',
                },
            });
            if (debug && subprocess.stdout) {
                subprocess.stdout.pipe(process.stdout);
            }
            await subprocess;
        }
        finally {
            await restoreUserConfig();
        }
        const defaultBuildManifest = await this.prepareBuildManifest();
        await this.buildRequestLambda(defaultBuildManifest);
    }
}
exports.default = Builder;
