"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const core_1 = require("@serverless/core");
const ramda_1 = require("ramda");
const utils_1 = require("./utils");
const outputsList = [
    'name',
    'hash',
    'description',
    'memory',
    'timeout',
    'code',
    'bucket',
    'shims',
    'handler',
    'runtime',
    'env',
    'role',
    'arn',
    'region',
];
const SUPPORTED_RUNTIMES = ['nodejs10.x', 'nodejs12.x'];
const LATEST_RUNTIME = SUPPORTED_RUNTIMES[1];
const defaults = {
    description: 'AWS Lambda Component',
    memory: 512,
    timeout: 10,
    code: process.cwd(),
    bucket: undefined,
    shims: [],
    handler: 'handler.hello',
    runtime: LATEST_RUNTIME,
    env: {},
    region: 'us-east-1',
};
class LambdaComponent extends core_1.Component {
    async default(inputs = {}) {
        this.context.status('Deploying');
        const config = ramda_1.mergeDeepRight(defaults, inputs);
        config.name = inputs.name || this.state.name || this.context.resourceId();
        this.context.debug(`Starting deployment of lambda ${config.name} to the ${config.region} region.`);
        const lambda = new aws_sdk_1.Lambda({
            region: config.region,
            credentials: this.context.credentials.aws,
        });
        const awsIamRole = await utils_1.load('@next-deploy/aws-iam-role', this);
        const outputsAwsIamRole = await awsIamRole.default(config.role);
        config.role = { arn: outputsAwsIamRole.arn };
        this.context.status('Packaging');
        this.context.debug(`Packaging lambda code from ${config.code}.`);
        config.zipPath = (await utils_1.pack(config.code, config.shims));
        config.hash = await core_1.utils.hashFile(config.zipPath);
        const prevLambda = await utils_1.getLambda({ lambda, ...config });
        if (!prevLambda) {
            this.context.status('Creating');
            this.context.debug(`Creating lambda ${config.name} in the ${config.region} region.`);
            const createResult = await utils_1.createLambda({ lambda, ...config });
            config.arn = createResult.arn;
            config.hash = createResult.hash;
        }
        else {
            config.arn = prevLambda.arn;
            if (utils_1.configChanged(prevLambda, config)) {
                if (prevLambda.hash !== config.hash) {
                    this.context.status(`Uploading code`);
                    this.context.debug(`Uploading ${config.name} lambda code.`);
                    await utils_1.updateLambdaCode({ lambda, ...config });
                }
                this.context.status(`Updating`);
                this.context.debug(`Updating ${config.name} lambda config.`);
                const updateResult = await utils_1.updateLambdaConfig({ lambda, ...config });
                config.hash = updateResult.hash;
            }
        }
        if (this.state.name && this.state.name !== config.name) {
            this.context.status(`Replacing`);
            await utils_1.deleteLambda({ lambda, name: this.state.name });
        }
        this.context.debug(`Successfully deployed lambda ${config.name} in the ${config.region} region.`);
        const outputs = ramda_1.pick(outputsList, config);
        this.state = outputs;
        await this.save();
        return outputs;
    }
    async publishVersion() {
        const { name, region, hash } = this.state;
        const lambda = new aws_sdk_1.Lambda({
            region: region,
            credentials: this.context.credentials.aws,
        });
        const { Version } = await lambda
            .publishVersion({
            FunctionName: name,
            CodeSha256: hash,
        })
            .promise();
        return { version: Version };
    }
    async remove() {
        this.context.status(`Removing`);
        if (!this.state.name) {
            this.context.debug(`Aborting removal. Function name not found in state.`);
            return;
        }
        const { name, region } = this.state;
        const lambda = new aws_sdk_1.Lambda({
            region: region,
            credentials: this.context.credentials.aws,
        });
        const awsIamRole = await utils_1.load('@next-deploy/aws-iam-role', this);
        await awsIamRole.remove();
        this.context.debug(`Removing lambda ${name} from the ${region} region.`);
        await utils_1.deleteLambda({ lambda, name: name });
        this.context.debug(`Successfully removed lambda ${name} from the ${region} region.`);
        const outputs = ramda_1.pick(outputsList, this.state);
        this.state = {};
        await this.save();
        return outputs;
    }
}
exports.default = LambdaComponent;
