"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = exports.pack = exports.configChanged = exports.getPolicy = exports.deleteLambda = exports.getLambda = exports.updateLambdaCode = exports.updateLambdaConfig = exports.createLambda = void 0;
const os_1 = require("os");
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const globby_1 = __importDefault(require("globby"));
const ramda_1 = require("ramda");
const fs_extra_1 = require("fs-extra");
const core_1 = require("@serverless/core");
const VALID_FORMATS = ['zip', 'tar'];
const isValidFormat = (format) => ramda_1.contains(format, VALID_FORMATS);
const packDir = async (inputDirPath, outputFilePath, include = [], exclude = [], prefix) => {
    const format = ramda_1.last(ramda_1.split('.', outputFilePath));
    if (!isValidFormat(format)) {
        throw new Error('Please provide a valid format. Either a "zip" or a "tar".');
    }
    const patterns = ['**/*'];
    if (!ramda_1.isNil(exclude)) {
        exclude.forEach((excludedItem) => patterns.push(`!${excludedItem}`));
    }
    const files = (await globby_1.default(patterns, { cwd: inputDirPath, dot: true }))
        .sort()
        .map((file) => ({
        input: path_1.default.join(inputDirPath, file),
        output: prefix ? path_1.default.join(prefix, file) : file,
    }));
    return new Promise((resolve, reject) => {
        const output = fs_extra_1.createWriteStream(outputFilePath);
        const archive = archiver_1.default(format, {
            zlib: { level: 9 },
        });
        output.on('open', () => {
            archive.pipe(output);
            files.forEach((file) => archive.append(fs_extra_1.createReadStream(file.input), {
                name: file.output,
                date: new Date(0),
            }));
            if (!ramda_1.isNil(include)) {
                include.forEach((file) => {
                    const stream = fs_extra_1.createReadStream(file);
                    archive.append(stream, {
                        name: path_1.default.basename(file),
                        date: new Date(0),
                    });
                });
            }
            archive.finalize();
        });
        archive.on('error', (err) => reject(err));
        output.on('close', () => resolve(outputFilePath));
    });
};
exports.createLambda = async ({ lambda, name, handler, memory, timeout, runtime, env, description, zipPath, bucket, role, }) => {
    const params = {
        FunctionName: name,
        Code: {},
        Description: description,
        Handler: handler,
        MemorySize: memory,
        Publish: true,
        Role: role.arn,
        Runtime: runtime,
        Timeout: timeout,
        Environment: {
            Variables: env,
        },
    };
    if (bucket) {
        params.Code.S3Bucket = bucket;
        params.Code.S3Key = path_1.default.basename(zipPath);
    }
    else {
        params.Code.ZipFile = await fs_extra_1.readFile(zipPath);
    }
    const res = await lambda.createFunction(params).promise();
    return { arn: res.FunctionArn, hash: res.CodeSha256 };
};
exports.updateLambdaConfig = async ({ lambda, name, handler, memory, timeout, runtime, env, description, role, }) => {
    const functionConfigParams = {
        FunctionName: name,
        Description: description,
        Handler: handler,
        MemorySize: memory,
        Role: role.arn,
        Runtime: runtime,
        Timeout: timeout,
        Environment: {
            Variables: env,
        },
    };
    const res = await lambda.updateFunctionConfiguration(functionConfigParams).promise();
    return { arn: res.FunctionArn, hash: res.CodeSha256 };
};
exports.updateLambdaCode = async ({ lambda, name, zipPath, bucket, }) => {
    const functionCodeParams = {
        FunctionName: name,
        Publish: true,
    };
    if (bucket) {
        functionCodeParams.S3Bucket = bucket;
        functionCodeParams.S3Key = path_1.default.basename(zipPath);
    }
    else {
        functionCodeParams.ZipFile = await fs_extra_1.readFile(zipPath);
    }
    const res = await lambda.updateFunctionCode(functionCodeParams).promise();
    return res.FunctionArn;
};
exports.getLambda = async ({ lambda, name, }) => {
    try {
        const res = await lambda
            .getFunctionConfiguration({
            FunctionName: name,
        })
            .promise();
        return {
            name: res.FunctionName,
            description: res.Description,
            timeout: res.Timeout,
            runtime: res.Runtime,
            role: {
                arn: res.Role,
            },
            handler: res.Handler,
            memory: res.MemorySize,
            hash: res.CodeSha256,
            env: res.Environment ? res.Environment.Variables : {},
            arn: res.FunctionArn,
        };
    }
    catch (e) {
        if (e.code === 'ResourceNotFoundException') {
            return null;
        }
        throw e;
    }
};
exports.deleteLambda = async ({ lambda, name, }) => {
    try {
        const params = { FunctionName: name };
        await lambda.deleteFunction(params).promise();
    }
    catch (error) {
        if (error.code !== 'ResourceNotFoundException') {
            throw error;
        }
    }
};
exports.getPolicy = ({ name, region, accountId, }) => {
    return {
        Version: '2012-10-17',
        Statement: [
            {
                Action: ['logs:CreateLogStream'],
                Resource: [`arn:aws:logs:${region}:${accountId}:log-group:/aws/lambda/${name}:*`],
                Effect: 'Allow',
            },
            {
                Action: ['logs:PutLogEvents'],
                Resource: [`arn:aws:logs:${region}:${accountId}:log-group:/aws/lambda/${name}:*:*`],
                Effect: 'Allow',
            },
        ],
    };
};
exports.configChanged = (prevLambda, lambda) => {
    const keys = ['description', 'runtime', 'role', 'handler', 'memory', 'timeout', 'env', 'hash'];
    const inputs = ramda_1.pick(keys, lambda);
    inputs.role = { arn: inputs.role.arn };
    const prevInputs = ramda_1.pick(keys, prevLambda);
    return ramda_1.not(ramda_1.equals(inputs, prevInputs));
};
exports.pack = async (code, shims = [], packDeps = true) => {
    if (core_1.utils.isArchivePath(code)) {
        return path_1.default.resolve(code);
    }
    let exclude = [];
    if (!packDeps) {
        exclude = ['node_modules/**'];
    }
    const outputFilePath = path_1.default.join(os_1.tmpdir(), `${Math.random().toString(36).substring(6)}.zip`);
    return packDir(code, outputFilePath, shims, exclude);
};
exports.load = async (path, that, name) => {
    const EngineComponent = await Promise.resolve().then(() => __importStar(require(path)));
    const component = new EngineComponent.default(`${that.id}.${name || EngineComponent.default.name}`, that.context.instance);
    await component.init();
    component.context.log = () => ({});
    component.context.status = () => ({});
    component.context.output = () => ({});
    return component;
};
