"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const ramda_1 = require("ramda");
const core_1 = require("@serverless/core");
const utils_1 = require("./lib/utils");
const uploadStaticAssets_1 = __importDefault(require("./lib/uploadStaticAssets"));
const syncStageStateDirectory_1 = __importDefault(require("./lib/syncStageStateDirectory"));
const defaults = {
    name: undefined,
    accelerated: true,
    region: 'us-east-1',
};
class AwsS3 extends core_1.Component {
    static uploadStaticAssets(options) {
        return uploadStaticAssets_1.default(options);
    }
    static syncStageStateDirectory(options) {
        return syncStageStateDirectory_1.default(options);
    }
    async default(inputs = {}) {
        var _a;
        const config = ramda_1.mergeDeepRight(defaults, inputs);
        this.context.status(`Deploying`);
        config.name = inputs.name || this.state.name || this.context.resourceId();
        this.context.debug(`Deploying bucket ${config.name} in region ${config.region}.`);
        const clients = utils_1.getClients(this.context.credentials.aws, config.region);
        await utils_1.ensureBucket(clients.regular, config.name, this.context.debug);
        if (config.accelerated) {
            if ((_a = config === null || config === void 0 ? void 0 : config.name) === null || _a === void 0 ? void 0 : _a.includes('.')) {
                throw new Error('Accelerated buckets must be DNS-compliant and must NOT contain periods');
            }
            this.context.debug(`Setting acceleration to "${config.accelerated}" for bucket ${config.name}.`);
            await utils_1.accelerateBucket(clients.regular, config.name, config.accelerated);
        }
        if (config.cors) {
            this.context.debug(`Setting cors for bucket ${config.name}.`);
            await utils_1.configureCors(clients.regular, config.name, config.cors);
        }
        const nameChanged = this.state.name && this.state.name !== config.name;
        if (nameChanged) {
            await this.remove();
        }
        this.state.name = config.name;
        this.state.region = config.region;
        this.state.accelerated = config.accelerated;
        this.state.url = `https://${config.name}.s3.amazonaws.com`;
        await this.save();
        this.context.debug(`Bucket ${config.name} was successfully deployed to the ${config.region} region.`);
        return this.state;
    }
    async remove() {
        this.context.status(`Removing`);
        if (!this.state.name) {
            this.context.debug(`Aborting S3 removal - bucket name not found in state.`);
            return;
        }
        const clients = utils_1.getClients(this.context.credentials.aws, this.state.region);
        this.context.debug(`Clearing bucket ${this.state.name} contents.`);
        await utils_1.clearBucket(this.state.accelerated ? clients.accelerated : clients.regular, this.state.name);
        this.context.debug(`Deleting bucket ${this.state.name} from region ${this.state.region}.`);
        await utils_1.deleteBucket(clients.regular, this.state.name);
        this.context.debug(`Bucket ${this.state.name} was successfully deleted from region ${this.state.region}.`);
        const outputs = {
            name: this.state.name,
            region: this.state.region,
            accelerated: this.state.accelerated,
        };
        this.state = {};
        await this.save();
        return outputs;
    }
    async upload(inputs = {}) {
        this.context.status('Uploading');
        const name = this.state.name || inputs.name;
        const region = this.state.region || inputs.region || defaults.region;
        if (!name) {
            throw Error('Unable to upload. Bucket name not found in state.');
        }
        this.context.debug(`Starting upload to bucket ${name} in region ${region}`);
        const clients = utils_1.getClients(this.context.credentials.aws, region);
        if (inputs.dir && (await core_1.utils.dirExists(inputs.dir))) {
            if (inputs.zip) {
                this.context.debug(`Packing and uploading directory ${inputs.dir} to bucket ${name}`);
                const defaultKey = Math.random().toString(36).substring(6);
                await utils_1.packAndUploadDir({
                    s3: this.state.accelerated ? clients.accelerated : clients.regular,
                    bucketName: name,
                    dirPath: inputs.dir,
                    key: inputs.key || `${defaultKey}.zip`,
                    cacheControl: inputs.cacheControl,
                });
            }
            else {
                this.context.debug(`Uploading directory ${inputs.dir} to bucket ${name}`);
                await utils_1.uploadDir(this.state.accelerated ? clients.accelerated : clients.regular, name, inputs.dir, inputs.cacheControl, { keyPrefix: inputs.keyPrefix });
            }
        }
        else if (inputs.file && (await core_1.utils.fileExists(inputs.file))) {
            this.context.debug(`Uploading file ${inputs.file} to bucket ${name}`);
            await utils_1.uploadFile({
                s3: this.state.accelerated ? clients.accelerated : clients.regular,
                bucketName: name,
                filePath: inputs.file,
                key: inputs.key || path_1.default.basename(inputs.file),
                cacheControl: inputs.cacheControl,
            });
            this.context.debug(`File ${inputs.file} uploaded with key ${inputs.key || path_1.default.basename(inputs.file)}`);
        }
    }
}
exports.default = AwsS3;
