"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMimeType = exports.filterOutDirectories = exports.readDirectoryFiles = exports.pathToPosix = exports.configureCors = exports.deleteBucket = exports.accelerateBucket = exports.clearBucket = exports.uploadFile = exports.packAndUploadDir = exports.uploadDir = exports.ensureBucket = exports.getClients = void 0;
const aws_sdk_1 = require("aws-sdk");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const klaw_sync_1 = __importDefault(require("klaw-sync"));
const klaw_1 = __importDefault(require("klaw"));
const mime_types_1 = __importDefault(require("mime-types"));
const s3_stream_upload_1 = __importDefault(require("s3-stream-upload"));
const ramda_1 = require("ramda");
const fs_extra_1 = require("fs-extra");
const archiver_1 = __importDefault(require("archiver"));
const core_1 = require("@serverless/core");
exports.getClients = (credentials, region) => {
    const params = {
        region,
        credentials,
    };
    return {
        regular: new aws_sdk_1.S3(params),
        accelerated: new aws_sdk_1.S3({ ...params, endpoint: `s3-accelerate.amazonaws.com` }),
    };
};
const bucketCreation = async (s3, Bucket) => {
    try {
        await s3.headBucket({ Bucket }).promise();
    }
    catch (e) {
        if (e.code === 'NotFound' || e.code === 'NoSuchBucket') {
            await core_1.utils.sleep(2000);
            return bucketCreation(s3, Bucket);
        }
        throw new Error(e);
    }
};
exports.ensureBucket = async (s3, name, debug) => {
    try {
        debug(`Checking if bucket ${name} exists.`);
        await s3.headBucket({ Bucket: name }).promise();
    }
    catch (e) {
        if (e.code === 'NotFound') {
            debug(`Bucket ${name} does not exist. Creating...`);
            await s3.createBucket({ Bucket: name }).promise();
            debug(`Bucket ${name} created. Confirming it's ready...`);
            await bucketCreation(s3, name);
            debug(`Bucket ${name} creation confirmed.`);
        }
        else if (e.code === 'Forbidden' && e.message === null) {
            throw Error(`Forbidden: Invalid credentials or this AWS S3 bucket name may already be taken`);
        }
        else if (e.code === 'Forbidden') {
            throw Error(`Bucket name "${name}" is already taken.`);
        }
        else {
            throw e;
        }
    }
};
exports.uploadDir = async (s3, bucketName, dirPath, cacheControl, options) => {
    const items = await new Promise((resolve, reject) => {
        try {
            resolve(klaw_sync_1.default(dirPath));
        }
        catch (error) {
            reject(error);
        }
    });
    const uploadItems = [];
    items.forEach((item) => {
        if (item.stats.isDirectory()) {
            return;
        }
        let key = path_1.default.relative(dirPath, item.path);
        if (options === null || options === void 0 ? void 0 : options.keyPrefix) {
            key = path_1.default.posix.join(options.keyPrefix, key);
        }
        if (path_1.default.sep === '\\') {
            key = key.replace(/\\/g, '/');
        }
        const itemParams = {
            Bucket: bucketName,
            Key: key,
            Body: fs_1.default.readFileSync(item.path),
            ContentType: mime_types_1.default.lookup(path_1.default.basename(item.path)) || 'application/octet-stream',
            CacheControl: cacheControl,
        };
        uploadItems.push(s3.upload(itemParams).promise());
    });
    await Promise.all(uploadItems);
};
exports.packAndUploadDir = async ({ s3, bucketName, dirPath, key, append = [], cacheControl, }) => {
    const ignore = (await core_1.utils.readFileIfExists(path_1.default.join(dirPath, '.slsignore'))) || [];
    return new Promise((resolve, reject) => {
        const archive = archiver_1.default('zip', {
            zlib: { level: 9 },
        });
        if (!ramda_1.isEmpty(append)) {
            append.forEach((file) => {
                const fileStream = fs_extra_1.createReadStream(file);
                archive.append(fileStream, { name: path_1.default.basename(file) });
            });
        }
        archive.glob('**/*', {
            cwd: dirPath,
            ignore,
        }, {});
        archive
            .pipe(s3_stream_upload_1.default(s3, {
            Bucket: bucketName,
            Key: key,
            CacheControl: cacheControl,
        }))
            .on('error', (err) => reject(err))
            .on('finish', () => resolve());
        archive.finalize();
    });
};
exports.uploadFile = async ({ s3, bucketName, filePath, key, cacheControl, }) => {
    return new Promise((resolve, reject) => {
        fs_1.default.createReadStream(filePath)
            .pipe(s3_stream_upload_1.default(s3, {
            Bucket: bucketName,
            Key: key,
            ContentType: mime_types_1.default.lookup(filePath) || 'application/octet-stream',
            CacheControl: cacheControl,
        }))
            .on('error', (err) => reject(err))
            .on('finish', () => resolve());
    });
};
exports.clearBucket = async (s3, bucketName) => {
    try {
        const data = await s3.listObjects({ Bucket: bucketName }).promise();
        const items = data.Contents || [];
        const promises = [];
        for (let i = 0; i < items.length; i += 1) {
            const deleteParams = { Bucket: bucketName, Key: items[i].Key };
            const delObj = s3.deleteObject(deleteParams).promise();
            promises.push(delObj);
        }
        await Promise.all(promises);
    }
    catch (error) {
        if (error.code !== 'NoSuchBucket') {
            throw error;
        }
    }
};
exports.accelerateBucket = async (s3, bucketName, accelerated) => {
    try {
        await s3
            .putBucketAccelerateConfiguration({
            AccelerateConfiguration: {
                Status: accelerated ? 'Enabled' : 'Suspended',
            },
            Bucket: bucketName,
        })
            .promise();
    }
    catch (e) {
        if (e.code === 'NoSuchBucket') {
            await core_1.utils.sleep(2000);
            return exports.accelerateBucket(s3, bucketName, accelerated);
        }
        throw e;
    }
};
exports.deleteBucket = async (s3, bucketName) => {
    try {
        await s3.deleteBucket({ Bucket: bucketName }).promise();
    }
    catch (error) {
        if (error.code !== 'NoSuchBucket') {
            throw error;
        }
    }
};
exports.configureCors = async (s3, bucketName, config) => {
    const params = { Bucket: bucketName, CORSConfiguration: config };
    try {
        await s3.putBucketCors(params).promise();
    }
    catch (e) {
        if (e.code === 'NoSuchBucket') {
            await core_1.utils.sleep(2000);
            return exports.configureCors(s3, bucketName, config);
        }
        throw e;
    }
};
exports.pathToPosix = (path) => path.replace(/\\/g, '/');
exports.readDirectoryFiles = (directory) => {
    const items = [];
    return new Promise((resolve, reject) => {
        klaw_1.default(directory.trim())
            .on('data', (item) => items.push(item))
            .on('end', () => {
            resolve(items);
        })
            .on('error', reject);
    });
};
exports.filterOutDirectories = (fileItem) => !fileItem.stats.isDirectory();
exports.getMimeType = (filePath) => mime_types_1.default.lookup(path_1.default.basename(filePath)) || 'application/octet-stream';
