"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = __importDefault(require("os"));
const chalk_1 = __importDefault(require("chalk"));
const ansi_escapes_1 = __importDefault(require("ansi-escapes"));
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const figures_1 = __importDefault(require("figures"));
const path_1 = __importDefault(require("path"));
const prettyoutput_1 = __importDefault(require("prettyoutput"));
const core_1 = require("@serverless/core");
const { red, green, blue, dim: grey } = chalk_1.default;
const STATE_ROOT = '.next-deploy';
class Context {
    constructor({ root, stateRoot, credentials, debug, entity, message }) {
        this.root = path_1.default.resolve(root) || process.cwd();
        this.stateRoot = stateRoot ? path_1.default.resolve(stateRoot) : path_1.default.join(this.root, STATE_ROOT);
        this.credentials = credentials || {};
        this.debugMode = debug || false;
        this.state = { id: core_1.utils.randomId() };
        this.id = this.state.id;
        this.outputs = {};
        this.metrics = {
            entity: entity || 'Component',
            lastDebugTime: undefined,
            useTimer: true,
            seconds: 0,
            status: {
                running: false,
                message: message || 'Running',
                loadingDots: '',
                loadingDotCount: 0,
            },
        };
        process.stdout.write(ansi_escapes_1.default.cursorHide);
        process.on('SIGINT', () => {
            if (this.isStatusEngineActive()) {
                return this.statusEngineStop('cancel');
            }
            process.exit(1);
        });
        setInterval(() => {
            this.metrics.seconds++;
        }, 1000);
    }
    async init() {
        const contextStatePath = path_1.default.join(this.stateRoot, `_.json`);
        if (await core_1.utils.fileExists(contextStatePath)) {
            this.state = await core_1.utils.readFile(contextStatePath);
        }
        else {
            await core_1.utils.writeFile(contextStatePath, this.state);
        }
        this.id = this.state.id;
        await this.setCredentials();
    }
    resourceId() {
        return `${this.id}-${core_1.utils.randomId()}`;
    }
    async readState(id) {
        const stateFilePath = path_1.default.join(this.stateRoot, `${id}.json`);
        if (await core_1.utils.fileExists(stateFilePath)) {
            return core_1.utils.readFile(stateFilePath);
        }
        return {};
    }
    async writeState(id, state) {
        const stateFilePath = path_1.default.join(this.stateRoot, `${id}.json`);
        await core_1.utils.writeFile(stateFilePath, state);
        return state;
    }
    async setCredentials() {
        const envVars = process.env;
        const providers = {};
        providers.aws = {};
        providers.aws.AWS_ACCESS_KEY_ID = 'accessKeyId';
        providers.aws.AWS_SECRET_ACCESS_KEY = 'secretAccessKey';
        providers.aws.AWS_REGION = 'region';
        const credentials = {};
        for (const provider in providers) {
            const providerEnvVars = providers[provider];
            for (const providerEnvVar in providerEnvVars) {
                if (!envVars.hasOwnProperty(providerEnvVar)) {
                    continue;
                }
                if (!credentials[provider]) {
                    credentials[provider] = {};
                }
                credentials[provider][providerEnvVars[providerEnvVar]] = envVars[providerEnvVar];
            }
        }
        this.credentials = credentials;
        return credentials;
    }
    close(reason, error) {
        process.stdout.write(ansi_escapes_1.default.cursorShow);
        if (!this.isStatusEngineActive()) {
            console.log();
            error && console.error(error);
            process.exit(0);
        }
        return this.statusEngineStop(reason, error);
    }
    getRelativeVerticalCursorPosition(contentString) {
        const base = 1;
        const terminalWidth = process.stdout.columns;
        const contentWidth = strip_ansi_1.default(contentString).length;
        const nudges = Math.ceil(Number(contentWidth) / Number(terminalWidth));
        return base + nudges;
    }
    async statusEngine() {
        this.renderStatus();
        await core_1.utils.sleep(100);
        if (this.isStatusEngineActive()) {
            return this.statusEngine();
        }
    }
    isStatusEngineActive() {
        return this.metrics.status.running;
    }
    statusEngineStart() {
        if (this.debugMode) {
            this.log();
        }
        this.metrics.status.running = true;
        return this.statusEngine();
    }
    statusEngineStop(reason, error) {
        this.metrics.status.running = false;
        let message = '';
        if (reason === 'error' && error) {
            message = red(`❌ ${error.stack || error.message}`);
        }
        else if (reason === 'cancel') {
            message = red('Cancelled ❌');
        }
        else if (reason === 'done') {
            message = green('Done ✔');
        }
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        this.log();
        let content = '';
        if (this.metrics.useTimer) {
            content += `${grey(`${this.metrics.seconds}s`)}`;
            content += ` ${blue(figures_1.default.pointerSmall)}`;
        }
        content += ` ${message}`;
        process.stdout.write(content);
        console.log(os_1.default.EOL);
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
        process.stdout.write(ansi_escapes_1.default.cursorShow);
        if (reason === 'error') {
            process.exit(1);
        }
        else {
            process.exit(0);
        }
    }
    renderStatus(status, entity) {
        if (!this.isStatusEngineActive()) {
            this.statusEngineStart();
        }
        if (status) {
            this.metrics.status.message = status;
        }
        if (entity) {
            this.metrics.entity = entity;
        }
        if (this.metrics.status.loadingDotCount === 0) {
            this.metrics.status.loadingDots = `.`;
        }
        else if (this.metrics.status.loadingDotCount === 2) {
            this.metrics.status.loadingDots = `..`;
        }
        else if (this.metrics.status.loadingDotCount === 4) {
            this.metrics.status.loadingDots = `...`;
        }
        else if (this.metrics.status.loadingDotCount === 6) {
            this.metrics.status.loadingDots = '';
        }
        this.metrics.status.loadingDotCount++;
        if (this.metrics.status.loadingDotCount > 8) {
            this.metrics.status.loadingDotCount = 0;
        }
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        console.log();
        let content = '';
        if (this.metrics.useTimer) {
            content += `${grey(this.metrics.seconds + 's')}`;
            content += ` ${blue(figures_1.default.pointerSmall)}`;
        }
        content += ` ${grey(this.metrics.status.message)}`;
        content += `${blue(this.metrics.status.loadingDots)}`;
        process.stdout.write(content);
        console.log();
        const startingPosition = this.getRelativeVerticalCursorPosition(content);
        process.stdout.write(ansi_escapes_1.default.cursorUp(startingPosition));
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
    }
    renderLog(msg) {
        if (!msg || msg == '') {
            console.log();
            return;
        }
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        console.log();
        console.log(msg);
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
    }
    renderDebug(msg) {
        if (!this.debugMode || !msg || msg == '') {
            return;
        }
        this.metrics.lastDebugTime = this.metrics.lastDebugTime || new Date().getTime();
        const now = new Date().getTime();
        const elapsedMs = now - this.metrics.lastDebugTime;
        const elapsedTimeSuffix = elapsedMs > 1000
            ? chalk_1.default.red(`(${Math.floor(elapsedMs / 1000)}s)`)
            : grey.bold(`(${elapsedMs}ms)`);
        this.metrics.lastDebugTime = now;
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        console.log(`${blue.bold(`DEBUG ${figures_1.default.line}`)} ${chalk_1.default.white(msg)} ${elapsedTimeSuffix}`);
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
    }
    renderError(errorProp) {
        let error;
        if (!errorProp) {
            return;
        }
        if (typeof errorProp === 'string') {
            error = new Error(errorProp);
        }
        else {
            error = errorProp;
        }
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        console.log();
        console.log(`${red('Error:')}`);
        console.log(` `, error);
        process.stdout.write(ansi_escapes_1.default.cursorLeft);
    }
    renderOutputs(outputs) {
        if (typeof outputs !== 'object' || Object.keys(outputs).length === 0) {
            return;
        }
        process.stdout.write(ansi_escapes_1.default.eraseDown);
        console.log();
        process.stdout.write(prettyoutput_1.default(outputs, {}, 2));
    }
    log(msg) {
        this.renderLog(msg);
    }
    debug(msg) {
        this.renderDebug(msg);
    }
    status(status, entity) {
        this.renderStatus(status, entity);
    }
}
exports.default = Context;
